package com.care.operate.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.care.bms.service.BmsMemberInfoService;
import com.care.common.annotation.Permission;
import com.care.common.exception.BDException;
import com.care.common.util.PageResult;
import com.care.common.util.Result;
import com.care.common.util.WebPageUtils;
import com.care.common.vo.PageReqVO;
import com.care.common.vo.UserLogindConvertVO;
import com.care.common.vo.memberinfo.MemberInfoVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @Author: stw
 * @Date: 2021/8/26
 * @Desc:
 */
@RestController
@Api(value = "OpMemberInfoController", tags = { "运营平台-后台用户管理" })
@Slf4j
@RequestMapping("/operate/memberinfo")
@Permission
public class OpMemberInfoController {

    @Autowired
    private BmsMemberInfoService bmsMemberInfoService;


    @GetMapping("/listMemberInfo")
    @ApiOperation(value = "用户列表分页--四期新增（和管理后台一样）")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "name", value = "用户名"),
            @ApiImplicitParam(paramType = "query", name = "phone", value = "电话"),
    })
    public PageResult<List<MemberInfoVO>> listMemberInfo(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                         @RequestParam(value = "name", required = false) String name,
                                                         @RequestParam(value = "phone", required = false) String phone,
                                                         PageReqVO pageReqVo){
        try {
            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            IPage<MemberInfoVO> pageResponse = this.bmsMemberInfoService.listCareMemberInfo(loginUser, name,phone,pageReqVo);
            if(pageResponse != null){
                return PageResult.success(pageResponse.getRecords(),pageResponse.getCurrent(),pageResponse.getSize(),pageResponse.getTotal());
            } else {
                return null;
            }

        }catch (BDException e) {
            log.error("用户列表查询-分页列表出现异常",e);
            return  PageResult.error(e.getMessage());
        } catch (Exception e) {
            log.error("用户管理: 用户列表查询出现异常",e);
            return PageResult.error( "获取列表失败");
        }
    }


    @PostMapping("/updateMemberInfoStatus/{memberId}/{status}")
    @ApiOperation(value = "修改用户状态，状态：0可用，1禁用   --四期新增（和管理后台一样）")
    public Result<Object> updateMemberInfoStatus(@RequestHeader("token") String token,
                                                 @PathVariable("memberId") Long memberId,
                                                 @PathVariable("status") String status) {
        if (this.bmsMemberInfoService.updateCareMemberInfoStatus(memberId, status)) {
            return Result.success("更新成功！");
        } else {
            return Result.error("更新失败！");
        }
    }


}
