package com.care.operate.controller;


import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.care.common.annotation.Permission;
import com.care.common.entity.CareStation;
import com.care.common.exception.BDException;
import com.care.common.service.CareStationService;
import com.care.common.util.PageResult;
import com.care.common.util.RandomCodeUtil;
import com.care.common.util.Result;

import com.care.common.util.WebPageUtils;
import com.care.common.vo.PageReqVO;
import com.care.common.vo.UserLogindConvertVO;
import com.care.common.vo.station.CareStationVO;
import com.care.operate.service.OpStationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;


/**
 * @Author: stw
 * @Date: 2021/5/26
 * @Desc:
 */
@RestController
@Api(value = "OpStationController", tags = { "运营平台-服务站管理" })
@Slf4j
@RequestMapping("/operate/station")
@Permission
public class OpStationController {

    @Resource
    private CareStationService careStationService;
    @Resource
    private OpStationService opStationService;

    @PostMapping("/addStation")
    @ApiOperation(value = "新增服务站 -- 四期新增")
    public Result<Object> addStation(HttpServletRequest request,@RequestHeader("token") String token,
                                    @RequestBody CareStationVO vo){
        try {
            QueryWrapper<CareStation> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(CareStation::getName,vo.getName());
            CareStation careStationDb = this.careStationService.getOne(queryWrapper);
            if(careStationDb != null ){
                return  Result.error("名称已经存在已存在！");
            }

            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            CareStation careStation = new CareStation();
            BeanUtil.copyProperties(vo,careStation);
            careStation.setOrgId(loginUser.getOrgId());
            careStation.setCode(getCode());
            boolean flag = this.careStationService.save(careStation);
            if (flag) {
                return  Result.success("新增成功！");
            }  else {
                return  Result.error("新增失败！");
            }

        }catch (BDException e) {
            log.error("新增服务站-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("新增服务站: 新增服务站出现异常",e);
            return Result.error("新增失败！");
        }
    }

    private String getCode(){
        String code = RandomCodeUtil.radomNumberCode(10);
        QueryWrapper<CareStation> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareStation::getCode,code);
        CareStation careStationDb1 = this.careStationService.getOne(queryWrapper);
        if(careStationDb1 == null) {
            return code;
        } else {
            return getCode();
        }
    }

    @PostMapping("/updateStation")
    @ApiOperation(value = "修改服务站 --四期新增")
    public Result<Object> updateStation(@RequestHeader("token") String token,
                                       @RequestBody CareStationVO vo){
        try {
            CareStation careStation = new CareStation();
            BeanUtil.copyProperties(vo,careStation);
            this.careStationService.updateById(careStation);
            return  Result.success("修改成功！");
        }catch (BDException e) {
            log.error("修改服务站-出现异常",e);
            return Result.error(e.getMessage());
        } catch (Exception e) {
            log.error("服务站管理: 修改服务站出现异常",e);
            return Result.error("修改失败！");
        }
    }

    @GetMapping("/getStationInfo/{stationId}")
    @ApiOperation(value = "服务站详情  --四期新增")
    public Result<CareStationVO> getStationInfo(@RequestHeader("token") String token,@PathVariable("stationId") Long stationId){
        CareStationVO vo = new CareStationVO();
        CareStation stationInfo = this.careStationService.getById(stationId);
        BeanUtil.copyProperties(stationInfo,vo);
        return  Result.success("查询成功！",vo);
    }

    @GetMapping("/listStation")
    @ApiOperation(value = "服务站列表分页  --四期新增")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(paramType = "query", name = "name", value = "服务站名称")
    })
    public PageResult<List<CareStationVO>> listStation(HttpServletRequest request, @RequestHeader(value = "token") String token,
                                                      @RequestParam(value = "name", required = false) String name,
                                                      PageReqVO pageReqVo){
        try {
            UserLogindConvertVO loginUser = WebPageUtils.getCurrentLoginedUser(request);
            IPage<CareStationVO> pageResponse = this.opStationService.listCareStation(name,loginUser ,pageReqVo);
            return PageResult.success(pageResponse.getRecords(),pageResponse.getCurrent(),pageResponse.getSize(),pageResponse.getTotal());
        }catch (BDException e) {
            log.error("服务站列表查询-分页列表出现异常",e);
            return  PageResult.error(e.getMessage());
        } catch (Exception e) {
            log.error("服务站管理: 服务站列表查询出现异常",e);
            return PageResult.error( "获取列表失败");
        }
    }
}
