package com.care.operate.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.care.common.entity.*;
import com.care.common.enums.DeviceActiveStatusEnum;
import com.care.common.enums.UserRoleEnum;
import com.care.common.enums.ValidStatusEnum;
import com.care.common.exception.BDException;
import com.care.common.service.*;
import com.care.common.util.MyBeanUtils;
import com.care.common.vo.PageReqVO;
import com.care.common.vo.UserLogindConvertVO;
import com.care.common.vo.device.DeviceVO;
import com.care.common.vo.sysuser.CareSysUserVO;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * (CareSysUser)表服务实现类
 *
 * @author makejava
 * @since 2021-05-21 00:08:38
 */
@Service
public class OpSysUserService {

    @Resource
    private CareOrganizationService careOrganizationService;
    @Resource
    private CareStationService careStationService;
    @Resource
    private CareSysUserService careSysUserService;
    @Resource
    private CareDeviceService careDeviceService;
    @Resource
    private CareEventOrderChambService careEventOrderChambService;
    @Resource
    private CareHouseService careHouseService;


    /**
     * 服务站账号列表分页
     * @param stationName
     * @return
             */
    public IPage<CareSysUserVO> listStationAccount(String stationName,UserLogindConvertVO loginUser, PageReqVO pageReqVo) {
        Long stationId = null;

        QueryWrapper<CareStation> queryWrapper0 = new QueryWrapper<>();
        queryWrapper0.lambda().like(StrUtil.isNotEmpty(stationName),CareStation::getName,stationName);
        CareStation careStation = careStationService.getOne(queryWrapper0);
        if (careStation != null){
            stationId = careStation.getId();
        }
        IPage<CareSysUser> page = new Page<>(pageReqVo.getCurrent(), pageReqVo.getPageSize());
        QueryWrapper<CareSysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda() .eq(stationId != null,CareSysUser::getStationId,stationId)
                .eq(CareSysUser::getOrgId,loginUser.getOrgId())
                .eq(CareSysUser::getRole,UserRoleEnum.STATION.getValue())
                .orderByAsc(CareSysUser::getName);

        IPage<CareSysUser> pageRes = this.careSysUserService.page(page, queryWrapper);
        IPage<CareSysUserVO> results = new Page<>(pageRes.getCurrent(),pageRes.getSize(),pageRes.getTotal());
        if(CollUtil.isNotEmpty(pageRes.getRecords())){
            List<CareSysUserVO> list = new ArrayList<>();
            pageRes.getRecords().forEach(item -> {
                CareSysUserVO  resVO = new CareSysUserVO();
                BeanUtils.copyProperties(item,resVO);
                //服务设备数量
                QueryWrapper<CareDevice> queryWrapper1 = new QueryWrapper<>();
                queryWrapper1.lambda().eq(CareDevice::getStationId,resVO.getStationId());
                resVO.setDevCount(this.careDeviceService.count(queryWrapper1));

                resVO.setValidStatusName(ValidStatusEnum.getCodeToName(resVO.getValidStatus()));
                list.add(resVO);
            });
            results.setRecords(list);
        }
        return results;
    }


    /**
     * 详情
     * @param id
     * @return
     */
    public CareSysUserVO getCareSysUserById(Long id){
        CareSysUser careSysUser = this.careSysUserService.getById(id);
        if (careSysUser!=null){
            CareSysUserVO resVO = new CareSysUserVO();
            BeanUtil.copyProperties(careSysUser,resVO);

            //服务设备列表
            QueryWrapper<CareDevice> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.lambda().eq(CareDevice::getStationId,resVO.getStationId());
            List<CareDevice> stationDeviceList = this.careDeviceService.list(queryWrapper1);

            if(CollUtil.isNotEmpty(stationDeviceList)){
                List<DeviceVO> stationDeviceVOList = new ArrayList<>();
                stationDeviceList.forEach(item ->{
                    DeviceVO deviceVO = new DeviceVO();
                    BeanUtil.copyProperties(item,deviceVO);
                    if(deviceVO.getHouseId() != null) {
                        CareHouse careHouse = careHouseService.getById(deviceVO.getHouseId());
                        if(careHouse != null) {
                            deviceVO.setHouseAddr(careHouse.getAddr());
                        }
                    }
                    deviceVO.setActiveStatusName(DeviceActiveStatusEnum.getCodeToName(deviceVO.getActiveStatus()));

                    stationDeviceVOList.add(deviceVO);
                });
                resVO.setStationDeviceVOList(stationDeviceVOList);

            }

            return resVO;
        }
        return null;
    }

    /**
     * 保存服务站账号
     * @param vo
     */
    @Transactional(rollbackFor = Exception.class)
    public int createCareSysUser(CareSysUserVO vo,UserLogindConvertVO loginUser){
        QueryWrapper<CareSysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CareSysUser::getPhone,vo.getPhone());
        if(careSysUserService.count(queryWrapper) > 0){
            return 1;
        }
        //保存服务站账号
        CareSysUser careSysUser = new CareSysUser();
        BeanUtil.copyProperties(vo,careSysUser);
        careSysUser.setCreateTime(new Date());
        careSysUser.setRole(UserRoleEnum.STATION.getValue());
        careSysUser.setOrgId(loginUser.getOrgId());

        this.careSysUserService.save(careSysUser);

        return 0;
    }

    /**
     * 修改服务站账号
     * @param vo
     */
    @Transactional(rollbackFor = Exception.class)
    public void updateCareSysUser(CareSysUserVO vo){
        //保存服务站账号
        CareSysUser careSysUser = this.careSysUserService.getById(vo.getId());
        BeanUtil.copyProperties(vo,careSysUser);
        careSysUser.setModifyTime(new Date());
        this.careSysUserService.updateById(careSysUser);
     }

}